@@ -37,7 +37,7 @@ def test_op(op_path, out_dir, test_id):
             break
 
 def validate(op_path, out_dir, test_ids=None, skip_ids=None, max_dtype_err=0,
-        rand_seed=0):
+        rand_seed=0, show_traceback=False):
     opschema.register(op_path)
     op = opschema.get(op_path)
 
@@ -51,7 +51,8 @@ def validate(op_path, out_dir, test_ids=None, skip_ids=None, max_dtype_err=0,
     elif isinstance(skip_ids, tuple):
         skip_ids = set(skip_ids)
 
-    return op.validate(out_dir, test_ids, skip_ids, max_dtype_err, rand_seed)
+    return op.validate(out_dir, test_ids, skip_ids, max_dtype_err, rand_seed,
+            show_traceback)
 
 def explain(op_path, include_inventory=False):
     return opschema.explain(op_path, include_inventory)
