@@ -1,13 +1,16 @@
 from opschema import genlib
+from opschema.complib import dilate, dilate_t, tconv, tconv_t
 
 def init_schema(op):
     op.add_index('b', 'batch', (1, 10))
-    op.add_index('i', 'input spatial', (2, 2))
-    op.add_index('k', 'input channel', (1, 1))
+    op.add_index('i', 'input spatial', 2)
+    op.add_index('k', 'input channel', 1)
     op.add_index('f', 'filter spatial', 'i')
-    op.add_index('l', 'output channel', (1, 1))
-    op.add_index('o', 'output spatial', 'i')
-    op.add_index('r', 'rate', (1, 1)) 
+    op.add_index('g', 'dilated filter spatial', 'i')
+    op.add_index('l', 'output channel', 1)
+    op.add_index('o', 'output spatial declared', 'i')
+    op.add_index('q', 'output spatial', 'i')
+    op.add_index('r', 'rate', 1) 
 
     op.arg_tensor('value', 'bik')
     op.arg_tensor('filters', 'flk')
@@ -15,7 +18,7 @@ def init_schema(op):
     op.arg_shape_int('rate', 'r')
     op.arg_option('padding', ('VALID', 'SAME'))
     op.arg_unchecked('name')
-    op.return_tensor('bol')
+    op.return_tensor('bql')
 
     op.gen_dims('b', 100)
     op.gen_dims('f', 100)
@@ -24,21 +27,17 @@ def init_schema(op):
     op.gen_dims('l', 30)
     op.gen_dims('r', 30)
 
-    def odims(i, f, r, padding):
-        if padding == 'VALID':
-            out = i - (f - 1) * r
-        else:
-            out = i
-        return out
+    op.comp_dims_cw('g', dilate, dilate_t, 'fr') 
 
-    def odims_t(i, f, r, padding):
+    def odims_gen(i, g, padding):
         if padding == 'VALID':
-            txt = f'{i} - ({f} - 1) * 2'
+            v = i + g - 1
+            yield v, v
         else:
-            txt = f'{i}'
-        return txt
+            yield i, i
 
-    # op.comp_dims_cw('o', odims, odims_t, 'ifr', 'padding')
+    op.gen_dims_calc('o', odims_gen, 'ig', 'padding')
+    op.comp_dims_cw('q', tconv, tconv_t, 'ig', 'padding')
 
     op.valid_dtypes('value', ('float',))
     op.equate_dtypes('filters', 'value')
